#include "ether.h"

// packet prototype
static BYTE packet[1000];
static incr = 0;

int main(){
  int i, j, h;
  WORD *pkt;
  WORD word;

  microblaze_enable_icache();

  print("\r\n\r\n---------------Starting-------------\r\n\r\n");
  diag();
    
  // reset the NIC card
  if(!init_etherne()){
    print("Ethernet NIC not detected");
    return;
  }
  
  // create a broadcast packet
  packet[0] = 0xff;
  packet[1] = 0xff;
  packet[2] = 0xff;
  packet[3] = 0xff;
  packet[4] = 0xff;
  packet[5] = 0xff;

  // source addr
  packet[6] = 0x01;
  packet[7] = 0x02;
  packet[8] = 0x03;
  packet[9] = 0x04;
  packet[10] = 0x05;
  packet[11] = 0x06;

  // length field
  packet[12] = 0x6a;
  packet[13] = 0x05;

  for(i=14; i<1000; i++)
    packet[i] = (BYTE)(i);

  pkt = (WORD *)packet;
  h=0;
  j=0;
  while(1){
    j=dmaWrite((pkt+h), (TXSTART<<8), 500);
    if(j){
      outnic(CMDR, 0x21); // abort the DMA operation
      delay(2);
      j=0;
      continue;
    }

    h++;
    if(h == 250){
      h = 0;
    }
    //for(i=0; i<10; i++);
    //delay(2);
  }

  print("\r\n\r\n---------------Done-------------\r\n");
  return 0;
}

// diagnostic that tests nic functionality
void diag(){
  int i;
  
  print("\r\n**************** Beginning of Diagnostic ****************\r\n\r\n");

  print("         Command Register Page Switching Test\r\n");
  print("Switching to page 1\r\n");
  outnic(CMDR, 0x61);

  print("Writing to and reading from 0x0D (value should be 0x4e): ");
  outnic(0x0D*2,0x4d);
  putnum(innic(0x0D*2));
  print("\r\n");

  print("Switching to page 0\r\n");
  outnic(CMDR, 0x21);

  print("Reading from reg offset 0x0D: ");
  putnum(innic(0x0D*2));
  print("\r\n");

  print("Switching to page 1\r\n");
  outnic(CMDR, 0x61);
  print("Reading from reg offset 0x0D (should be 0x4e): ");
  putnum(innic(0x0D*2));
  print("\r\n\r\n");

  print("         Default Value Test\r\n");
  print("Switching to page 0\r\n");
  outnic(CMDR,21);
  print("Reading from 0x16 (value should be 0x15): ");
  putnum(innic(0x16*2));
  print("\r\n");
  print("Reading from 0x12 (value should be 0x0c): ");
  putnum(innic(0x12*2));
  print("\r\n");
  print("Reading from 0x13 (value should be 0x12): ");
  putnum(innic(0x13*2));
  print("\r\n\r\n**************** End of Diagnostic ****************\r\n\r\n");
}

int init_etherne()
{
  outnic(NE_RESET, innic(NE_RESET));  // Do reset
  delay(2);
  if ((innic(ISR) & 0x80) == 0)       // Report if failed
  {
    print("  Ethernet card failed to reset!\r\n");
    return 0;
  }
  else
  {
    print("Ethernet card reset successful...");
    resetnic(); // Reset Ethernet card,
    print("Ethernet card intialization complete!\r\n");
  }
  return 1;
}

void resetnic(){
  int i, count;

  outnic(CMDR, 0x21);  // Abort and DMA and stop the NIC
  delay(10);

  outnic(DCR, DCRVAL);   // Set word-wide access

  outnic(RBCR0, 0x00); // Clear the count regs
  outnic(RBCR1, 0x00);

  outnic(IMR, 0x00);   // Mask completion irq
  outnic(ISR, 0xFF);   // clear interrupt status register

  outnic(RCR, 0x20);   // 0x20  Set to monitor mode
  outnic(TCR, 0x02);  // put nic in normal operation

  // Set Rx start, Rx stop, Boundary and TX start regs
  outnic(PSTART, RXSTART);
  outnic(PSTOP, RXSTOP);
  outnic(BNRY, (BYTE)(RXSTOP-1));
  outnic(TPSR, TXSTART);
  
  outnic(ISR, 0xFF);   // clear interrupt status register
  outnic(IMR, 0x00);   // Mask completion irq

  // put nic in start mode
  outnic(CMDR, 0x22); // start nic
  outnic(TCR, 0x00);  // put nic in normal operation
}

/*
BYTE send(WORD *packet, WORD len){
  int i,j;
  WORD counter;
  

  while((innic(CMDR)&0x04)); // block if busy

  counter = len>>1;

  outnic(RSAR0, 0x00); // set DMA starting address
  outnic(RSAR1, TXSTART);

  outnic(ISR, 0xFF); // clear ISR

  outnic(RBCR0, (len&0xff)); // set Remote DMA Byte Count
  outnic(RBCR1, (len>>8));

  outnic(TBCR0, (len&0xff)); // set Transmit Byte Count
  outnic(TBCR1, (len>>8));

  outnic(CMDR, 0x12); // start the DMA write

  // change order of MS/LS since DMA
  // writes LS byte in 15-8, and MS byte in 7-0
  for(i=0; i<counter; i++){
    //outnic(DATAPORT, (data[i]<<8|data[i]>>8)); // write data
    outnic(DATAPORT, packet[i]);
  }
  
  // abort packet send if the DMA doesn't complete
  if(!(innic(ISR)&0x40))
    return 1;

  outnic(CMDR, 0x24);
  j=10000;
  while(j-->0 && !(innic(ISR)&0x02));

  // abort packet send, if it doesn't send correctly
  if(!j)
    return 1;

  outnic(ISR,0xFF); // reset interrupt status register

  return 0;
}
*/
/*
BYTE dmaWrite(WORD *data, WORD addr, WORD len){
  int i,j;
  WORD counter;
  BYTE odd = 0;

  //outnic(RSAR0, 0x00);
  //outnic(RSAR1, TXSTART);
  //outnic(RBCR0, 0x02);
  //outnic(RBCR1, 0x00);
  //outnic(CMDR, 0x0a);

  counter = len>>1;
  //counter = len;

  outnic(RSAR0, (addr&0xff)); // set DMA starting address
  outnic(RSAR1, (addr>>8));

  outnic(ISR, 0xFF); // clear ISR

  outnic(RBCR0, (len&0xff)); // set Remote DMA Byte Count
  outnic(RBCR1, (len>>8));

  outnic(TBCR0, (len&0xff)); // set Transmit Byte Count
  outnic(TBCR1, (len>>8));

  outnic(CMDR, 0x12); // start the DMA write

  // change order of MS/LS since DMA
  // writes LS byte in 15-8, and MS byte in 7-0
  for(i=0,j=0; i<counter;){
    outnic(DATAPORT, (data[i]<<8)|(data[i]>>8));
    j++;
    i=j>>1;
  }
  incr++;

  //j=10000;
  //while(j>0 && !(innic(ISR)&0x40)){
  //  j--;
  //}
  //if(!j)
  //return 1;
  
  
  if(!(innic(ISR)&0x40)){
    //print("DMA did not finish! ");
    //putnum(innic(ISR));
    //print("\r\n");
    return 1;
  }
  else{
    //print("DMA did finish! ");
    //putnum(innic(ISR));
    //print(" ");
  }
  
  outnic(TPSR, TXSTART); // set Transmit Page Start Register
  outnic(CMDR, 0x24); // start transmission
  
  j=100;
  while(j-->0 && !(innic(ISR)&0x02));
  if(!j){
    //print("Transmission did not work! ");
    //putnum(innic(ISR));
    //print("\r\n");
    return 1;
  }
  //print("Transmission did work! ");
  //putnum(innic(ISR));
  //print("\r\n");
  
  //print("Sent Packet: ");
  //putnum(innic(ISR));
  //print("\r\n");
  outnic(ISR,0xFF);

  return 0;
}
*/
void delay(int mult){
  int i;
  int delay = 1000000*mult;
  for(i=0; i<delay; i++);
}
